@@ -34,8 +34,8 @@ def transform_function(func_tree):
             test.body = assigns + test.body
 
         else:
-            raise TypeError("Don't know how to match %s" \
-                % (codegen.to_source(cond).strip() or cond))
+            raise TypeError("Don't know how to match %s"
+                            % (codegen.to_source(cond).strip() or cond))
 
     func_tree.body = lmap(wrap_tail_expr, func_tree.body)
     func_tree.body.append(make_raise(N('Mismatch')))
@@ -47,23 +47,38 @@ def transform_function(func_tree):
     # print(codegen.to_source(func_tree))
 
 
-def destruct_to_tests_and_assigns(topic, pattern):
+NAMED_CONSTS = {'None': None, 'True': True, 'False': False}
+
+def destruct_to_tests_and_assigns(topic, pattern, names=None):
+    if names is None:
+        names = {}
+
     if isinstance(pattern, (Num, Str)):
         return [make_eq(topic, pattern)], []
     elif isinstance(pattern, Name):
-        return [], [make_assign(pattern.id, topic)]
+        if pattern.id in NAMED_CONSTS:
+            return [make_op(Is, topic, pattern)], []
+        elif pattern.id == '_':
+            return [], []
+        else:
+            if pattern.id in names:
+                return [make_eq(topic, names[pattern.id])], []
+            else:
+                names[pattern.id] = topic
+                return [], [make_assign(pattern.id, topic)]
     elif isinstance(pattern, NameConstant):
         return [make_op(Is, topic, pattern)], []
     elif isinstance(pattern, Compare) and len(pattern.ops) == 1 and isinstance(pattern.ops[0], Is):
-        return [make_call('isinstance', topic, pattern.comparators[0])], \
-               [make_assign(pattern.left.id, topic)]
+        left_tests, left_assigns = destruct_to_tests_and_assigns(topic, pattern.left, names)
+        test = make_call('isinstance', topic, pattern.comparators[0])
+        return left_tests + [test], left_assigns
     elif isinstance(pattern, (List, Tuple, Dict)):
         elts = getattr(pattern, 'elts', []) or getattr(pattern, 'values', [])
         coll_tests = [
             make_call('isinstance', topic, N(pattern.__class__.__name__.lower())),
             make_eq(make_call('len', topic), len(elts))
         ]
-        tests, assigns = subscript_tests_and_assigns(topic, pattern)
+        tests, assigns = subscript_tests_and_assigns(topic, pattern, names)
         return coll_tests + tests, assigns
     elif isinstance(pattern, BinOp) and isinstance(pattern.op, Add) \
          and isinstance(pattern.left, (List, Tuple)) and isinstance(pattern.right, Name):
@@ -72,24 +87,29 @@ def destruct_to_tests_and_assigns(topic, pattern):
             make_op(GtE, make_call('len', topic), len(pattern.left.elts)),
         ]
         coll_assigns = [
-            make_assign(pattern.right.id,
-                        Subscript(ctx=Load(),
-                                  value=topic,
-                                  slice=Slice(lower=V(len(pattern.left.elts)), upper=None, step=None)))
+            make_assign(
+                pattern.right.id,
+                Subscript(
+                    ctx   = Load(),
+                    value = topic,
+                    slice = Slice(lower=V(len(pattern.left.elts)), upper=None, step=None)
+                )
+            )
         ]
-        tests, assigns = subscript_tests_and_assigns(topic, pattern.left)
+        tests, assigns = subscript_tests_and_assigns(topic, pattern.left, names)
         return coll_tests + tests, assigns + coll_assigns
     else:
-        raise TypeError("Don't know how to match %s" \
-            % (codegen.to_source(pattern).strip() or pattern))
+        raise TypeError("Don't know how to match %s"
+                        % (codegen.to_source(pattern).strip() or pattern))
 
 
-def subscript_tests_and_assigns(topic, pattern):
+def subscript_tests_and_assigns(topic, pattern, names):
     tests = []
     assigns = []
-    items = enumerate(pattern.elts) if hasattr(pattern, 'elts') else zip(pattern.keys, pattern.values)
+    items = enumerate(pattern.elts) if hasattr(pattern, 'elts') else \
+            zip(pattern.keys, pattern.values)
     for key, elt in items:
-        t, a = destruct_to_tests_and_assigns(make_subscript(topic, key), elt)
+        t, a = destruct_to_tests_and_assigns(make_subscript(topic, key), elt, names)
         tests.extend(t)
         assigns.extend(a)
     return tests, assigns
